module cpu
(
    input               clock,
    input               reset_n,
    input               locked,
    /* verilator lint_off UNDRIVEN */
    output      [19:0]  address,
    input       [ 7:0]  i_data,
    output  reg [ 7:0]  o_data,
    output  reg         we
);

`include "decl.v"

always @(posedge clock)
// Либо сброс, либо инициализация
if (!reset_n || !locked) begin cs <= 16'hF000; ip <= 16'hFFF0; t <= init; end
else case (t)

    // Первый такт сбрасывает инструкцию перед запуском
    init: begin

        t       <= fetch;
        we      <= 1'b0;
        segment <= ds;
        segover <= 1'b0;
        rep     <= 2'b00;

    end

// Считывание опкода
fetch: begin

    /* verilator lint_off CASEX */
    casex (i_data)

        // Обработка префиксов
        8'h26: begin segment <= es; segover <= 1'b1; end
        8'h2E: begin segment <= cs; segover <= 1'b1; end
        8'h36: begin segment <= ss; segover <= 1'b1; end
        8'h3E: begin segment <= ds; segover <= 1'b1; end
        8'hF2, 8'hF3: begin rep <= i_data[1:0]; end
        8'hF0, 8'h64, 8'h65, 8'h66, 8'h67, 8'h0F: begin /* ничего не делать */ end
        // Запись опкода или частичное исполнение
        default: t <= exec;

    endcase

    ip      <= ip + 1'b1;
    opcode  <= i_data;

end

endcase

endmodule
